@@ -6,8 +6,8 @@
 from composer.metrics.map import MAP
 from composer.metrics.metrics import CrossEntropy, Dice, LossMetric, MIoU
 from composer.metrics.nlp import (BinaryF1Score, HFCrossEntropy, InContextLearningLMAccuracy, InContextLearningMetric,
-                                  InContextLearningMultipleChoiceAccuracy, LanguageCrossEntropy, MaskedAccuracy,
-                                  Perplexity)
+                                  InContextLearningMultipleChoiceAccuracy, LanguageCrossEntropy, LanguagePerplexity,
+                                  MaskedAccuracy, Perplexity)
 
 __all__ = [
     'MAP',
@@ -20,6 +20,7 @@ __all__ = [
     'HFCrossEntropy',
     'LanguageCrossEntropy',
     'MaskedAccuracy',
+    'LanguagePerplexity',
     'InContextLearningLMAccuracy',
     'InContextLearningMultipleChoiceAccuracy',
     'InContextLearningMetric',
