@@ -10,6 +10,7 @@ import json
 import logging
 import tempfile
 import textwrap
+import warnings
 from collections import UserDict
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Type, Union
@@ -23,6 +24,7 @@ from composer.utils import MissingConditionalImportError, get_file, import_objec
 
 if TYPE_CHECKING:
     import transformers
+    from transformers import PretrainedConfig
     from transformers.models.auto.auto_factory import _BaseAutoModelClass
 
 log = logging.getLogger(__name__)
@@ -41,7 +43,8 @@ class HuggingFaceModel(ComposerModel):
             .. note:: If the tokenizer is provided, its config will be saved in the composer checkpoint, and it can be reloaded
                 using :meth:`HuggingFaceModel.hf_from_composer_checkpoint`. If the tokenizer is not provided here, it will not be saved in the composer checkpoint.
         use_logits (bool, optional): If True, the model's output logits will be used to calculate validation metrics. Else, metrics will be inferred from the HuggingFaceModel directly. Default: ``False``
-        metrics (list[Metric], optional): list of torchmetrics to apply to the output of `validate`. Default: ``None``.
+        metrics (list[Metric], optional): list of torchmetrics to apply to the output of `eval_forward` during training. If ``eval_metrics`` is ``None``, these will also be used as ``eval_metrics``.  Default: ``None``.
+        eval_metrics (list[Metric], optional): list of torchmetrics to compute on the eval_dataloader, or be accessible to :class:`Evaluator`s. Default: ``None``.
         shift_labels (bool, optional): If True, the batch's labels will be shifted before being used to calculate metrics. This should be set to true for CausalLM models and false otherwise. If not specified, `shift_labels` will be set automatically based on the model class name. Default: ``None``.
 
         .. note:: To ensure correct behavior, set `shift_labels` manually if using a custom model (i.e., if `model` is not
@@ -66,6 +69,7 @@ class HuggingFaceModel(ComposerModel):
                                            transformers.PreTrainedTokenizerFast]] = None,
                  use_logits: Optional[bool] = False,
                  metrics: Optional[List[Metric]] = None,
+                 eval_metrics: Optional[List[Metric]] = None,
                  shift_labels: Optional[bool] = None) -> None:
         try:
             import transformers
@@ -78,6 +82,7 @@ class HuggingFaceModel(ComposerModel):
         super().__init__()
         self.model = model
         self.config = model.config
+        self.model_forward_args = inspect.getfullargspec(self.model.forward).args
         self.tokenizer = tokenizer
 
         if self.tokenizer is None:
@@ -95,9 +100,12 @@ class HuggingFaceModel(ComposerModel):
         self.train_metrics: Optional[Dict] = None
         self.val_metrics: Optional[Dict] = None
 
-        if metrics:
+        if metrics is not None:
             self.train_metrics = {metric.__class__.__name__: metric for metric in metrics}
-            self.val_metrics = {metric.__class__.__name__: metric for metric in metrics}
+            if eval_metrics is None:
+                self.val_metrics = {metric.__class__.__name__: metric for metric in metrics}
+            else:
+                self.val_metrics = {metric.__class__.__name__: metric for metric in eval_metrics}
 
         self.labels: Optional[torch.Tensor] = None  # set in eval_forward() if exists
 
@@ -149,15 +157,19 @@ class HuggingFaceModel(ComposerModel):
         .. testcode::
 
             hf_model, hf_tokenizer = HuggingFaceModel.hf_from_composer_checkpoint('composer-hf-checkpoint.pt')
+            # At this point, hf_model is randomly initialized
             composer_model = HuggingFaceModel(hf_model, hf_tokenizer)
             trainer = Trainer(model=composer_model,
                               train_dataloader=train_dataloader,
                               save_filename='composer-hf-checkpoint-2.pt',
                               max_duration='1ep',
-                              save_folder='./')
+                              save_folder='./',
+                              load_path='composer-hf-checkpoint.pt')
+            # At this point, the weights have been loaded from the composer checkpoint into hf_model
 
         Args:
-            checkpoint_path (str): Path to the composer checkpoint, can be a local path, ``http(s)://`` url, or ``s3://`` uri
+            checkpoint_path (str): Path to the composer checkpoint, can be a local path, or a remote path beginning with ``s3://``, or another backend
+                supported by :meth:`composer.utils.maybe_create_object_store_from_uri`.
             model_instantiation_class (Union[Type[:class:`transformers.PreTrainedModel`], Type[:class:`transformers.AutoModel`], str]), optional):
                 Class to use to create the HuggingFace model. Defaults to the model class used in the original checkpoint. If this argument is
                 a HuggingFace auto class (e.g. :class:`transformers.AutoModel` or :class:`transformers.AutoModelForSequenceClassification`), the ``from_config`` method will be used,
@@ -193,20 +205,14 @@ class HuggingFaceModel(ComposerModel):
         get_file(checkpoint_path, str(local_checkpoint_save_location))
 
         # load the state dict in
-        loaded_state_dict = torch.load(local_checkpoint_save_location)
+        loaded_state_dict = torch.load(local_checkpoint_save_location, map_location='cpu')
 
         hf_state = loaded_state_dict['state']['integrations']['huggingface']
         hf_model_state = hf_state['model']
         hf_tokenizer_state = hf_state['tokenizer']
 
-        hf_config_dict = hf_model_state['config']['content']
-        # Update the config with any extra args needed
-        hf_config_dict.update(model_config_kwargs)
-        # JSON keys need to be converted back to ints, huggingface does not auto convert them along this code path
-        if 'id2label' in hf_config_dict:
-            hf_config_dict['id2label'] = {int(k): v for k, v in hf_config_dict['id2label'].items()}
+        loaded_config = get_hf_config_from_composer_state_dict(loaded_state_dict, config_overrides=model_config_kwargs)
 
-        loaded_config = transformers.AutoConfig.from_pretrained(hf_config_dict['_name_or_path'], **hf_config_dict)
         if model_instantiation_class is not None:
             # If the instantiation class is explicitly provided, use it
             # If a string is provided, attempt to import the class it refers to
@@ -270,7 +276,7 @@ class HuggingFaceModel(ComposerModel):
     def forward(self, batch):
         if isinstance(batch, dict) or isinstance(batch, UserDict):
             # Further input validation is left to the huggingface forward call
-            batch = {k: v for k, v in batch.items() if k in inspect.getfullargspec(self.model.forward).args}
+            batch = {k: v for k, v in batch.items() if k in self.model_forward_args}
             output = self.model(**batch)  # type: ignore (thirdparty)
         else:
             raise ValueError(
@@ -290,6 +296,16 @@ class HuggingFaceModel(ComposerModel):
             # pop labels first to avoid computing loss
             self.labels = batch.pop('labels')
 
+            # HF encoder decoder models like T5 expect either decoder_input_ids or labels,
+            # so we add decoder_input_ids to the batch if it is missing
+            if self.model.config.is_encoder_decoder and 'decoder_input_ids' not in batch:
+                if hasattr(self.model, 'prepare_decoder_input_ids_from_labels'):
+                    batch['decoder_input_ids'] = self.model.prepare_decoder_input_ids_from_labels(labels=self.labels)
+                else:
+                    raise RuntimeError(
+                        'Encoder decoder models require that either decoder_input_ids is present in the batch'
+                        ' or that the model has a prepare_decoder_input_ids_from_labels method.')
+
             if self.shift_labels or batch.get('mode', None) == 'icl_task':
                 assert self.labels is not None
                 # HF CausalLM models internally shift labels before computing loss, so we do the same here
@@ -301,9 +317,9 @@ class HuggingFaceModel(ComposerModel):
             if self.config.use_return_dict:
                 output = output['logits']
             else:
-                # logits are at index 0 in the output tuple
-                # because we have popped labels, so no loss is present in the tuple
-                output = output[0]
+                # if loss was computed (cached outputs from forward), loss is at index 0 and logits are at index 1
+                # if loss was not computed (no cached outputs during eval), loss is not present and logits are at index 0
+                output = output[1] if len(output[0].shape) == 0 else output[0]
 
             # if we are in the single class case, then remove the classes dimension
             if output.shape[1] == 1:
@@ -367,6 +383,9 @@ class HuggingFaceModel(ComposerModel):
         return {'model': model_output, 'tokenizer': tokenizer_output}
 
     def add_eval_metrics(self, evaluator):
+        warnings.warn(
+            DeprecationWarning('The add_eval_metrics method is deprecated and will be removed in a future release. '
+                               'Please pass in `eval_metrics` directly to the constructor.'))
         evaluator_metrics = {m: METRIC_DEFAULT_CTORS[m]() for m in evaluator.metric_names}
         if self.val_metrics is not None:
             self.val_metrics.update(evaluator_metrics)
@@ -384,3 +403,109 @@ def _is_registered_causal_lm(model: transformers.PreTrainedModel) -> bool:
                                             conda_channel='conda-forge') from e
     causal_lm_classes = list(MODEL_FOR_CAUSAL_LM_MAPPING.values())
     return any([isinstance(model, causal_lm_class) for causal_lm_class in causal_lm_classes])
+
+
+def get_hf_config_from_composer_state_dict(state_dict: Dict[str, Any],
+                                           config_overrides: Optional[Dict[str, Any]] = None) -> 'PretrainedConfig':
+    """Get a HuggingFace config from a composer state dict with overrides applied
+
+    Args:
+        state_dict (Dict[str, Any]): The state dict to get the config from
+        config_overrides (Dict[str, Any], optional): Any overrides to apply to the config
+
+    Returns:
+        transformers.PretrainedConfig: The HuggingFace config
+    """
+    try:
+        import transformers
+    except ImportError as e:
+        raise MissingConditionalImportError(extra_deps_group='nlp',
+                                            conda_package='transformers',
+                                            conda_channel='conda-forge') from e
+
+    if config_overrides is None:
+        config_overrides = {}
+
+    hf_config_dict = state_dict['state']['integrations']['huggingface']['model']['config']['content']
+    # Update the config with any extra args needed
+    hf_config_dict.update(config_overrides)
+    # JSON keys need to be converted back to ints, huggingface does not auto convert them along this code path
+    if 'id2label' in hf_config_dict:
+        hf_config_dict['id2label'] = {int(k): v for k, v in hf_config_dict['id2label'].items()}
+
+    return transformers.AutoConfig.from_pretrained(hf_config_dict['_name_or_path'], **hf_config_dict)
+
+
+def write_huggingface_pretrained_from_composer_checkpoint(
+        checkpoint_path: Union[Path, str],
+        output_folder: Union[Path, str],
+        local_checkpoint_save_location: Optional[Union[Path, str]] = None) -> None:
+    """Write a ``config.json`` and ``pytorch_model.bin``, like :meth:`transformers.PreTrainedModel.from_pretrained` expects, from a composer checkpoint
+
+    .. note:: This function will not work properly if you used surgery algorithms when you trained your model. In that case you will want to
+        load the model weights using the Composer :class:`~composer.Trainer` with the ``load_path`` argument.
+
+    .. testsetup::
+
+        import torch
+
+        dataset = RandomTextClassificationDataset(size=16, use_keys=True)
+        train_dataloader = torch.utils.data.DataLoader(dataset, batch_size=8)
+        eval_dataloader = torch.utils.data.DataLoader(dataset, batch_size=8)
+
+        import transformers
+        from composer.models import HuggingFaceModel
+        from composer.trainer import Trainer
+
+        hf_model = transformers.AutoModelForSequenceClassification.from_pretrained('prajjwal1/bert-tiny', num_labels=2)
+        hf_tokenizer = transformers.AutoTokenizer.from_pretrained('prajjwal1/bert-tiny')
+        composer_model = HuggingFaceModel(hf_model, tokenizer=hf_tokenizer, metrics=[], use_logits=True)
+        trainer = Trainer(model=composer_model,
+                            train_dataloader=train_dataloader,
+                            save_filename='composer-hf-checkpoint.pt',
+                            max_duration='1ep',
+                            save_folder='./')
+        trainer.fit()
+        trainer.close()
+
+    Example:
+
+    .. testcode::
+
+        from composer.models import write_huggingface_pretrained_from_composer_checkpoint
+
+        write_huggingface_pretrained_from_composer_checkpoint('composer-hf-checkpoint.pt', './hf-save-pretrained-output')
+        loaded_model = transformers.AutoModelForSequenceClassification.from_pretrained('./hf-save-pretrained-output')
+
+    Args:
+        checkpoint_path (Union[Path, str]): Path to the composer checkpoint, can be a local path, or a remote path beginning with ``s3://``, or another backend
+            supported by :meth:`composer.utils.maybe_create_object_store_from_uri`.
+        output_folder (Union[Path, str]): Path to the folder to write the output to. Must be a local path.
+        local_checkpoint_save_location (Optional[Union[Path, str]], optional): If specified, where to save the checkpoint file to locally.
+                                                                                If the input ``checkpoint_path`` is already a local path, this will be a symlink.
+                                                                                Defaults to None, which will use a temporary file.
+    """
+    try:
+        import transformers
+        del transformers
+    except ImportError as e:
+        raise MissingConditionalImportError(extra_deps_group='nlp',
+                                            conda_package='transformers',
+                                            conda_channel='conda-forge') from e
+
+    # default local path to a tempfile if path is not provided
+    if local_checkpoint_save_location is None:
+        tmp_dir = tempfile.TemporaryDirectory()
+        local_checkpoint_save_location = Path(tmp_dir.name) / 'local-composer-checkpoint.pt'
+
+    # download the checkpoint file
+    get_file(str(checkpoint_path), str(local_checkpoint_save_location))
+
+    composer_state_dict = torch.load(local_checkpoint_save_location, map_location='cpu')
+
+    config = get_hf_config_from_composer_state_dict(composer_state_dict)
+    config.save_pretrained(output_folder)
+
+    weights_state_dict = composer_state_dict['state']['model']
+    torch.nn.modules.utils.consume_prefix_in_state_dict_if_present(weights_state_dict, prefix='model.')
+    torch.save(weights_state_dict, Path(output_folder) / 'pytorch_model.bin')
